//
// VIIR.cpp
//
// This file contains pre-compiled test rules for the IIIE Inference Engine

// RULES FOR WORLD


// Standard includes: 
#include <standard.h>

// Context dependent includes: 
#include "VIIS.h"

// Rules: 
rule_ptr VIIR_StartGameRule_ptr;
rule_ptr VIIR_AlliedFleetHailCheckRule_ptr;
rule_ptr VIIR_Enemy_Fleet_Destroyed_ptr;
rule_ptr VIIR_Players_Fleet_Destroyed_ptr;
rule_ptr VIIR_DisplayMainTitleRule_ptr;
rule_ptr VIIR_EndTitleRule_ptr;
rule_ptr VIIR_StartCinematicRule_ptr;



//
// STARTGAMERULE
//

// Left-hand side of StartGameRule rule
int VIIR_StartGameRule_left_hand()
{
	return ( NEED_TO_BEGIN_MISSION );
}

// Right hand side of StartGameRule rule
void VIIR_StartGameRule_right_hand()
{
	 Start_Mission();
}



//
// ALLIEDFLEETHAILCHECKRULE
//

// Left-hand side of AlliedFleetHailCheckRule rule
int VIIR_AlliedFleetHailCheckRule_left_hand()
{
	return ( ! MISSION_END && MISSION_BEGUN && tAlliedHailCheck );
}

// Right hand side of AlliedFleetHailCheckRule rule
void VIIR_AlliedFleetHailCheckRule_right_hand()
{
	 Check_Allied_Fleet_Hails();
}



//
// ENEMY_FLEET_DESTROYED
//

// Left-hand side of Enemy_Fleet_Destroyed rule
int VIIR_Enemy_Fleet_Destroyed_left_hand()
{
	return ( ALWAYS_EVALUATE && g_pSI->CountUnits ( 2 ) <= 0 && ! MISSION_END && MISSION_BEGUN );
}

// Right hand side of Enemy_Fleet_Destroyed rule
void VIIR_Enemy_Fleet_Destroyed_right_hand()
{
	 Mission_Success();
}



//
// PLAYERS_FLEET_DESTROYED
//

// Left-hand side of Players_Fleet_Destroyed rule
int VIIR_Players_Fleet_Destroyed_left_hand()
{
	return ( ALWAYS_EVALUATE && g_pSI->CountUnits ( 1 ) <= 0 && ! MISSION_END && MISSION_BEGUN );
}

// Right hand side of Players_Fleet_Destroyed rule
void VIIR_Players_Fleet_Destroyed_right_hand()
{
	 Mission_Fail();
}



//
// DISPLAYMAINTITLERULE
//

// Left-hand side of DisplayMainTitleRule rule
int VIIR_DisplayMainTitleRule_left_hand()
{
	return ( tDisplayMainTitle );
}

// Right hand side of DisplayMainTitleRule rule
void VIIR_DisplayMainTitleRule_right_hand()
{
	 Display_Main_Title();
}



//
// ENDTITLERULE
//

// Left-hand side of EndTitleRule rule
int VIIR_EndTitleRule_left_hand()
{
	return ( tRemoveMainTitle );
}

// Right hand side of EndTitleRule rule
void VIIR_EndTitleRule_right_hand()
{
	 Remove_Main_Title();
}



//
// STARTCINEMATICRULE
//

// Left-hand side of StartCinematicRule rule
int VIIR_StartCinematicRule_left_hand()
{
	return ( PLAY_SIMPLE_CINEMATIC );
}

// Right hand side of StartCinematicRule rule
void VIIR_StartCinematicRule_right_hand()
{
	 Play_Cinematic();
}



//
// VIIR_rules_init
//
// Initialize the data for this rule set and add the rules to KBs.
void VIIR_rules_init()
{
	// Add rules into the character's knowledge base's inference engine.
	VIIR_StartGameRule_ptr = 
		WORLD.GetIIIE().create_rule(
			VIIR_StartGameRule_left_hand, 
			VIIR_StartGameRule_right_hand, 
			"StartGameRule"); 

	VIIR_AlliedFleetHailCheckRule_ptr = 
		WORLD.GetIIIE().create_rule(
			VIIR_AlliedFleetHailCheckRule_left_hand, 
			VIIR_AlliedFleetHailCheckRule_right_hand, 
			"AlliedFleetHailCheckRule"); 

	VIIR_Enemy_Fleet_Destroyed_ptr = 
		WORLD.GetIIIE().create_rule(
			VIIR_Enemy_Fleet_Destroyed_left_hand, 
			VIIR_Enemy_Fleet_Destroyed_right_hand, 
			"Enemy_Fleet_Destroyed"); 

	VIIR_Players_Fleet_Destroyed_ptr = 
		WORLD.GetIIIE().create_rule(
			VIIR_Players_Fleet_Destroyed_left_hand, 
			VIIR_Players_Fleet_Destroyed_right_hand, 
			"Players_Fleet_Destroyed"); 

	VIIR_DisplayMainTitleRule_ptr = 
		WORLD.GetIIIE().create_rule(
			VIIR_DisplayMainTitleRule_left_hand, 
			VIIR_DisplayMainTitleRule_right_hand, 
			"DisplayMainTitleRule"); 

	VIIR_EndTitleRule_ptr = 
		WORLD.GetIIIE().create_rule(
			VIIR_EndTitleRule_left_hand, 
			VIIR_EndTitleRule_right_hand, 
			"EndTitleRule"); 

	VIIR_StartCinematicRule_ptr = 
		WORLD.GetIIIE().create_rule(
			VIIR_StartCinematicRule_left_hand, 
			VIIR_StartCinematicRule_right_hand, 
			"StartCinematicRule"); 

	// Add dependencies for variable NEED_TO_BEGIN_MISSION.
	NEED_TO_BEGIN_MISSION.add_dependent_rule(VIIR_StartGameRule_ptr);

	// Add dependencies for variable MISSION_BEGUN.
	MISSION_BEGUN.add_dependent_rule(VIIR_AlliedFleetHailCheckRule_ptr);
	MISSION_BEGUN.add_dependent_rule(VIIR_Enemy_Fleet_Destroyed_ptr);
	MISSION_BEGUN.add_dependent_rule(VIIR_Players_Fleet_Destroyed_ptr);

	// Add dependencies for variable MISSION_END.
	MISSION_END.add_dependent_rule(VIIR_AlliedFleetHailCheckRule_ptr);
	MISSION_END.add_dependent_rule(VIIR_Enemy_Fleet_Destroyed_ptr);
	MISSION_END.add_dependent_rule(VIIR_Players_Fleet_Destroyed_ptr);

	// Add dependencies for variable PLAY_SIMPLE_CINEMATIC.
	PLAY_SIMPLE_CINEMATIC.add_dependent_rule(VIIR_StartCinematicRule_ptr);

	// Add dependencies for variable ALWAYS_EVALUATE.
	ALWAYS_EVALUATE.add_dependent_rule(VIIR_Enemy_Fleet_Destroyed_ptr);
	ALWAYS_EVALUATE.add_dependent_rule(VIIR_Players_Fleet_Destroyed_ptr);

	// Add dependencies for variable tAlliedHailCheck.
	tAlliedHailCheck.add_dependent_rule(VIIR_AlliedFleetHailCheckRule_ptr);

	// Add dependencies for variable tDisplayMainTitle.
	tDisplayMainTitle.add_dependent_rule(VIIR_DisplayMainTitleRule_ptr);

	// Add dependencies for variable tRemoveMainTitle.
	tRemoveMainTitle.add_dependent_rule(VIIR_EndTitleRule_ptr);
}



//
// VIIR_rules_trash
//
// Free the data for this rule set and remove the rules from KB.
void VIIR_rules_trash()
{

	// Remove dependencies for the NEED_TO_BEGIN_MISSION variable.
	NEED_TO_BEGIN_MISSION.remove_dependent_rule(VIIR_StartGameRule_ptr);

	// Remove dependencies for the MISSION_BEGUN variable.
	MISSION_BEGUN.remove_dependent_rule(VIIR_AlliedFleetHailCheckRule_ptr);
	MISSION_BEGUN.remove_dependent_rule(VIIR_Enemy_Fleet_Destroyed_ptr);
	MISSION_BEGUN.remove_dependent_rule(VIIR_Players_Fleet_Destroyed_ptr);

	// Remove dependencies for the MISSION_END variable.
	MISSION_END.remove_dependent_rule(VIIR_AlliedFleetHailCheckRule_ptr);
	MISSION_END.remove_dependent_rule(VIIR_Enemy_Fleet_Destroyed_ptr);
	MISSION_END.remove_dependent_rule(VIIR_Players_Fleet_Destroyed_ptr);

	// Remove dependencies for the PLAY_SIMPLE_CINEMATIC variable.
	PLAY_SIMPLE_CINEMATIC.remove_dependent_rule(VIIR_StartCinematicRule_ptr);

	// Remove dependencies for the ALWAYS_EVALUATE variable.
	ALWAYS_EVALUATE.remove_dependent_rule(VIIR_Enemy_Fleet_Destroyed_ptr);
	ALWAYS_EVALUATE.remove_dependent_rule(VIIR_Players_Fleet_Destroyed_ptr);

	// Remove dependencies for the tAlliedHailCheck variable.
	tAlliedHailCheck.remove_dependent_rule(VIIR_AlliedFleetHailCheckRule_ptr);

	// Remove dependencies for the tDisplayMainTitle variable.
	tDisplayMainTitle.remove_dependent_rule(VIIR_DisplayMainTitleRule_ptr);

	// Remove dependencies for the tRemoveMainTitle variable.
	tRemoveMainTitle.remove_dependent_rule(VIIR_EndTitleRule_ptr);

	// Trash rules from the Knowledge Base's Inference engine
	WORLD.GetIIIE().trash_rule(VIIR_StartGameRule_ptr);
	WORLD.GetIIIE().trash_rule(VIIR_AlliedFleetHailCheckRule_ptr);
	WORLD.GetIIIE().trash_rule(VIIR_Enemy_Fleet_Destroyed_ptr);
	WORLD.GetIIIE().trash_rule(VIIR_Players_Fleet_Destroyed_ptr);
	WORLD.GetIIIE().trash_rule(VIIR_DisplayMainTitleRule_ptr);
	WORLD.GetIIIE().trash_rule(VIIR_EndTitleRule_ptr);
	WORLD.GetIIIE().trash_rule(VIIR_StartCinematicRule_ptr);
}



//
// VIIR_rules_entry
//
// Entry routine for the VIIR rules library
int WINAPI DllMain(HANDLE hModule,
	DWORD ul_reason_for_call,
	LPVOID lpReserved)
{
	switch (ul_reason_for_call)
	{
	case DLL_PROCESS_ATTACH:
		VIIR_rules_init();
		break;
	case DLL_PROCESS_DETACH:
		VIIR_rules_trash();
		break;
	case DLL_THREAD_ATTACH:
		break;
	case DLL_THREAD_DETACH:
		break;
	default: break;
	}

	return TRUE;
}
